package webui

import (
	"strings"

	"git.openpdu.org/OpenPDU/openpdu/config"
	"git.openpdu.org/OpenPDU/openpdu/events"
	"git.openpdu.org/OpenPDU/openpdu/ups"
	"gopkg.in/macaron.v1"
)

func upsPage(ctx *macaron.Context) {
	ctx.Data["host"] = config.GetString("Ups.Host")
	ctx.Data["port"] = config.GetInt("Ups.Port")
	ctx.Data["upsname"] = config.GetString("Ups.Name")
	ctx.Data["username"] = config.GetString("Ups.Username")
	ctx.Data["password"] = config.GetString("Ups.Password")

	if ups.Connected {
		ctx.Data["serverstatus"] = "Connected"
	} else {
		ctx.Data["serverstatus"] = "Disconnected"
	}

	ctx.Data["upsstatus"] = ups.Vars["ups.status"].Value
	ctx.Data["upsmfr"] = ups.Vars["device.mfr"].Value
	ctx.Data["upsmodel"] = ups.Vars["device.model"].Value
	ctx.Data["upsserial"] = ups.Vars["device.serial"].Value
	ctx.Data["upsload"] = ups.Vars["ups.load"].Value
	ctx.Data["upscharge"] = ups.Vars["battery.charge"].Value

	ctx.HTML(200, "ups")
}

type UPSPostForm struct {
	Host     string `form:"host" binding:"Required"`
	Port     int    `form:"port" binding:"Required"`
	UpsName  string `form:"upsname" binding:"Required"`
	Username string `form:"username"`
	Password string `form:"password"`
}

func upsPost(ctx *macaron.Context, f UPSPostForm) {
	config.Set("Ups.Host", strings.ToLower(strings.TrimSpace(f.Host)))
	config.Set("Ups.Port", f.Port)
	config.Set("Ups.Name", strings.TrimSpace(f.UpsName))
	config.Set("Ups.Username", strings.TrimSpace(f.Username))
	config.Set("Ups.Password", f.Password)
	config.WriteConfig()

	events.FireEvent("ups_config_changed")

	upsPage(ctx)
}
